/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#ifndef _ATDATETIMEORDERIVED_HPP
#define _ATDATETIMEORDERIVED_HPP

#include <xercesc/util/XMLString.hpp>
#include "DateOrTimeType.hpp"

#include <pathan/Pathan.hpp>

class ATDecimalOrDerived;
class ATDurationOrDerived;
class Timezone;

class PATHAN_EXPORT ATDateTimeOrDerived : public DateOrTimeType
{

public:

  /* Get the name of the primitive type (basic type) of this type
   * (ie "decimal" for xs:decimal) */
  virtual const XMLCh* getPrimitiveTypeName() const = 0;

  /* Get the namespace URI for this type */
  virtual const XMLCh* getTypeURI() const = 0;

  /* Get the name of this type  (ie "integer" for xs:integer) */
  virtual const XMLCh* getTypeName() const = 0;

  /* returns the XMLCh* (lexical) representation of this type */
  virtual const XMLCh* asString(const DynamicContext* context) const = 0;

  /** Releases the memory used by this Item */
  virtual void release() const = 0;

  /* returns true if the two objects represent the same date,
   * false otherwise */
  virtual bool equals(const AnyAtomicType* target, const DynamicContext* context) const = 0;


  /**
   * Returns true if and only if this date is greater than the given date. 
   * The order relation on date values is the order relation on their 
   * starting instants.
   */
  virtual bool greaterThan(const ATDateTimeOrDerived* date, const DynamicContext* context) const = 0;

  /**
   * Returns true if and only if this date is less than the given date. 
   * The order relation on date values is the order relation on their
   * starting instants.
   */
  virtual bool lessThan(const ATDateTimeOrDerived* date, const DynamicContext* context) const = 0;
 
  /** 
   * Returns an integer representing the year component  of this object
   */
  virtual const ATDecimalOrDerived* getYears() const = 0;

  /** 
   * Returns an integer representing the month component  of this object
   */
  virtual const ATDecimalOrDerived* getMonths() const = 0;

  /** 
   * Returns an integer representing the day component  of this object
   */
  virtual const ATDecimalOrDerived* getDays() const = 0;

  /** 
   * Returns an integer representing the hour component of this object
   */
  virtual const ATDecimalOrDerived* getHours() const = 0;

  /** 
   * Returns an integer representing the minute component of this object
   */
  virtual const ATDecimalOrDerived* getMinutes() const = 0;

  /** 
   * Returns a decimal representing the second component of this object
   */
  virtual const ATDecimalOrDerived* getSeconds() const = 0;

  /** 
   * Returns a timezone object representing the timezone component of this object
   */
  virtual const Timezone* getTimezone() const = 0;

  /**
   * Returns true if the timezone is defined for this object, false otherwise.
   */
  virtual bool hasTimezone() const = 0;

  /**
   * Setter for timezone.  Overrides the current timezone. (Not to be 
   * confused with addTimezone().
   */
  virtual const ATDateTimeOrDerived* setTimezone(const Timezone* timezone, const DynamicContext* context) const = 0;

  virtual const ATDateTimeOrDerived* normalize(const DynamicContext* context) const = 0;

  /**
   * Returns an ATDateTimeOrDerived with a timezone added to it
   */
  virtual const ATDateTimeOrDerived* addTimezone(const ATDurationOrDerived* timezone, const DynamicContext* context) const = 0;

  /**
   * Returns a date with the given yearMonthDuration added to it
   */
  virtual const ATDateTimeOrDerived* addYearMonthDuration(const ATDurationOrDerived* yearMonth,  const DynamicContext* context) const = 0;

  /**
   * Returns a date with the given dayTimeDuration added to it
   */
  virtual const ATDateTimeOrDerived* addDayTimeDuration(const ATDurationOrDerived* dayTime,  const DynamicContext* context) const = 0;
  
  /**
   * Returns a date with the given yearMonthDuration subtracted from it
   */
  virtual const ATDateTimeOrDerived* subtractYearMonthDuration(const ATDurationOrDerived* yearMonth,  const DynamicContext* context) const = 0;

  /**
   * Returns a date with the given dayTimeDuration subtracted from it
   */
  virtual const ATDateTimeOrDerived* subtractDayTimeDuration(const ATDurationOrDerived* dayTime,  const DynamicContext* context) const = 0;

  /**
   * Returns a dayTimeDuration corresponding to the difference between this
   * and the given ATDateTimeOrDerived*
   */
  virtual const ATDurationOrDerived* subtractDateTimeAsDayTimeDuration(const ATDateTimeOrDerived* date,  const DynamicContext* context) const = 0;

    /**
   * Returns a dayTimeDuration corresponding to the difference between this
   * and the given ATDateTimeOrDerived*
   */
  virtual const ATDurationOrDerived* subtractDateTimeAsYearMonthDuration(const ATDateTimeOrDerived* date, const DynamicContext* context) const = 0;


  virtual AnyAtomicType::AtomicObjectType getPrimitiveTypeIndex() const = 0;
  
protected: 

  /* Protected constructor */
  ATDateTimeOrDerived(XPath2MemoryManager* memMgr): DateOrTimeType(memMgr) {};

};

#endif //  _ATDATETIMEORDERIVED_HPP
